@@ -9,7 +9,7 @@ except ImportError:
 
 class GraphAPI(object):
 
-    def __init__(self, oauth_token=False):
+    def __init__(self, oauth_token=False, url='https://graph.facebook.com'):
         """
         Initialize GraphAPI with an OAuth access token.
 
@@ -17,6 +17,7 @@ class GraphAPI(object):
         """
         self.oauth_token = oauth_token
         self.session = requests.session()
+        self.url = url.strip('/')
 
     def get(self, path='', page=False, **options):
         """
@@ -33,6 +34,9 @@ class GraphAPI(object):
 
         response = self._query('GET', path, options, page)
 
+        if isinstance(response, Exception):
+            raise response
+
         if response is False:
             raise self.FacebookError('Could not get "%s".' % path)
 
@@ -112,27 +116,36 @@ class GraphAPI(object):
         )
 
         for response, request in zip(responses, requests):
-            if response:
-                data = json.loads(response['body'])
-            else:
+
+            # Facilitate for empty Graph API responses.
+            #
+            # https://github.com/jgorset/facepy/pull/30
+            if not response:
                 yield None
                 continue
 
-            if not response['code'] == 200:
-                yield self.FacebookError(
-                    '{type} error on {method} "{path}": {msg}.'.format(
-                        type = data['error']['type'],
-                        method = request['method'],
-                        path = request['relative_url'],
-                        msg = data['error']['message']
-                    ), data['error']['code']
-                )
+            data = self._parse(response['body'])
 
+            if isinstance(data, Exception):
+                data.request = request
+
+                yield data
                 continue
 
             yield data
 
-    def _query(self, method, path, data={}, page=False):
+    def fql(self, query):
+        """
+        Use FQL to powerfully extract data from Facebook.
+
+        :param query: A FQL query or FQL multiquery ({'query_name': "query",...})
+
+        See `Facebook's FQL documentation <http://developers.facebook.com/docs/reference/fql/>`_
+        for an exhaustive list of details.
+        """
+        return self._query('GET', 'fql?%s' % urlencode({'q': query}))
+
+    def _query(self, method, path, data=None, page=False):
         """
         Fetch an object from the Graph API and parse the output, returning a tuple where the first item
         is the object yielded by the Graph API and the second is the URL for the next page of results, or
@@ -143,6 +156,7 @@ class GraphAPI(object):
         :param data: A dictionary of HTTP GET parameters (for GET requests) or POST data (for POST requests).
         :param page: A boolean describing whether to return an iterator that iterates over each page of results.
         """
+        data = data or {}
 
         def load(method, url, data):
             if method in ['GET', 'DELETE']:
@@ -192,10 +206,11 @@ class GraphAPI(object):
 
         # Convert option lists to comma-separated values.
         for key in data:
-            if isinstance(data[key], list) and all([isinstance(item, basestring) for item in data[key]]):
+            if (isinstance(data[key], list) or isinstance(data[key], tuple) or isinstance(data[key], set)) \
+                and all([isinstance(item, basestring) for item in data[key]]):
                 data[key] = ','.join(data[key])
 
-        url = 'https://graph.facebook.com/%s' % path
+        url = '%s/%s' % (self.url, path)
 
         if self.oauth_token:
             data['access_token'] = self.oauth_token
@@ -233,19 +248,20 @@ class GraphAPI(object):
         if type(data) is dict:
             if 'error' in data:
                 error = data['error']
-                error_type = error.get('type')
 
-                if error_type == "OAuthException":
+                if error.get('type') == "OAuthException":
                     exception = self.OAuthError
                 else:
                     exception = self.FacebookError
 
-                raise exception(
+                return exception(
                     error.get('message'),
                     error.get('code', None)
                 )
+
+            # Facebook occasionally reports errors in its legacy error format.
             if 'error_msg' in data:
-                raise self.FacebookError(
+                return self.FacebookError(
                     data.get('error_msg'),
                     data.get('error_code', None)
                 )
